@@ -91,6 +91,7 @@ class PulpError(Exception):
     """
     Pulp Exception Class
     """
+    pass
 
 def setConfigInformation(**keywords):
     """
@@ -446,7 +447,16 @@ class LpVariable(LpElement):
 
 
 class LpAffineExpression(odict.OrderedDict):
-    """A linear combination of LP variables"""
+    """
+    A linear combination of LP variables
+    
+    An LpAffine Expression can be initalised with the following
+    e = None: an empty Expression
+    e = dict: gives an expression with the values being the coefficents of the keys
+    e = list: equivalent to dict.items()
+    e = LpElement: an expression of length 1 with the coefficent 1
+    e = other: the constant is intialised as e
+    """
     #to remove illegal characters from the names
     trans = string.maketrans("-+[] ","_____")
     def setname(self,name):
@@ -472,6 +482,9 @@ class LpAffineExpression(odict.OrderedDict):
         elif isinstance(e,dict):
             self.constant = constant
             super(LpAffineExpression,self).__init__(e.items())
+        elif isinstance(e, list):
+            self.constant = constant
+            super(LpAffineExpression,self).__init__(e)
         elif isinstance(e,LpElement):
             self.constant = 0
             super(LpAffineExpression,self).__init__( [(e, 1)])
@@ -717,6 +730,7 @@ class LpConstraint(LpAffineExpression):
         if rhs is not None:
             self.constant = - rhs
         self.sense = sense
+        self.modified = True
 
     def __str__(self):
         s = LpAffineExpression.__str__(self, 0)
@@ -755,6 +769,13 @@ class LpConstraint(LpAffineExpression):
             s += sl + ns + "\n"
         return s
 
+    def changeRHS(self, RHS):
+        """
+        alters the RHS of a constraint so that it can be modefied in a resolve
+        """
+        self.constant = -RHS
+        self.modified = True
+        
     def __repr__(self):
         s = LpAffineExpression.__repr__(self)
         if self.sense is not None:
@@ -858,6 +879,70 @@ class LpConstraint(LpAffineExpression):
         if self.sense == LpConstraintEQ: return abs(val) <= eps
         else: return val * self.sense >= - eps
 
+    def makeElasticSubProblem(self, *args, **kwargs):
+        """
+        Builds an elastic subproblem by adding variables to a hard constraint
+
+        uses FixedElasticSubProblem
+        """
+        return FixedElasticSubProblem(self, *args, **kwargs)
+
+class LpFractionConstraint(LpConstraint):
+    """
+    Creates a constraint that enforces a fraction requirement a/b = c
+    """
+    def __init__(self, numerator, denominator = None, sense = LpConstraintEQ, 
+                 RHS = 1.0, name = None,
+                 complement = None):
+        """
+        creates a fraction Constraint to model constraints of
+        the nature 
+        numerator/denominator {==, >=, <=} RHS        
+        numerator/(numerator + complement) {==, >=, <=} RHS
+        
+        @param numerator: the top of the fraction
+        @param denominator: as described above
+        @param sense: the sense of the relation of the constraint
+        @param RHS: the target fraction value
+        @param complement: as described above
+        """
+        self.numerator = numerator
+        if denominator is None and complement is not None:
+            self.complement = complement
+            self.denominator = numerator + complement
+        elif denominator is not None and complement is None:
+            self.denominator = denominator
+            self.complement = denominator - numerator
+        else:
+            raise PulpError, 'one, and only one, of numerator and " +\
+                "complement must be specified'
+        lhs = self.numerator - RHS * self.denominator
+        LpConstraint.__init__(self, lhs,
+              sense = sense, rhs = 0, name = name)
+        self.RHS = RHS
+        
+    def findLHSValue(self):
+        """
+        Determines the value of the fraction in the constraint after solution
+        """
+        if abs(value(self.denominator))>= EPS:
+            return value(self.numerator)/value(self.denominator)
+        else:
+            if abs(value(self.numerator))<= EPS:
+                #zero divided by zero will return 1
+                return 1.0
+            else:
+                raise ZeroDivisionError
+            
+    def makeElasticSubProblem(self, *args, **kwargs):
+        """
+        Builds an elastic subproblem by adding variables and splitting the 
+        hard constraint
+
+        uses FractionElasticSubProblem
+        """
+        return FractionElasticSubProblem(self, *args, **kwargs)
+
 class LpConstraintVar(LpElement):
     """A Constraint that can be treated as a variable when constructing
     a LpProblem by columns
@@ -869,7 +954,8 @@ class LpConstraintVar(LpElement):
                                        rhs = rhs , e = e)
         
     def addVariable(self, var, coeff):
-        """Adds a variable to the constraint with the
+        """
+        Adds a variable to the constraint with the
         activity coeff
         """
         self.constraint.addterm(var, coeff)
@@ -877,7 +963,7 @@ class LpConstraintVar(LpElement):
     def value(self):
         return self.constraint.value()
 
-class LpProblem:
+class LpProblem(object):
     """An LP Problem"""
     def __init__(self, name = "NoName", sense = LpMinimize):
         """
@@ -1097,25 +1183,30 @@ class LpProblem:
             raise TypeError, "Can only add LpConstraintVar, LpConstraint, LpAffineExpression or True objects"
         return self
     
-    def extend(self, contraints):
+    def extend(self, other):
         """
         extends an LpProblem by adding constraints either from a dictionary
         a tuple or another LpProblem object.
 
         For dictionaries the constraints will be named with the keys
-        For tuples an unquie name will be generated
+        For tuples an unique name will be generated
         For LpProblems the name of the problem will be added to the constraints
         name
         """
-        if isinstance(contraints, dict):
-            for name in contraints:
-                self.constraints[name] = contraints[name]
-        elif isinstance(contraints, LpProblem):
-            for name,c in contraints.constraints.iteritems():
-                c.name = contraints.name + name
+        if isinstance(other, dict):
+            for name in other:
+                self.constraints[name] = other[name]
+        elif isinstance(other, LpProblem):
+            if other.name[:4] =='None':
+                #no constraint name so make one
+                other.alterName(self.unusedConstraintName())
+            othername = other.name
+            for name,c in other.constraints.iteritems():
+                c.name = othername + name
                 self.addConstraint(c)
+            self.objective += other.objective
         else:
-            for c in contraints:
+            for c in other:
                 if isinstance(c,tuple):
                     name = c[0]
                     c = c[1]
@@ -1408,14 +1499,15 @@ class LpProblem:
         self.solver = solver
         return statuses
     
-    def resolve(self, solver = None):
-        """ resolves an Problem using the same solver as previously
+    def resolve(self, solver = None, **kwargs):
+        """ 
+        resolves an Problem using the same solver as previously
         """
         if not(solver): solver = self.solver
         if self.resolveOK:
-            return self.solver.actualResolve(self)
+            return self.solver.actualResolve(self, **kwargs)
         else:
-            return self.solve(solver = solver)
+            return self.solve(solver = solver, **kwargs)
         
     def setSolver(self,solver = LpSolverDefault):
         """Sets the Solver for this problem useful if you are using 
@@ -1426,8 +1518,186 @@ class LpProblem:
     def setInitial(self,values):
         self.initialValues = values
     
+class FixedElasticSubProblem(LpProblem):
+    """
+    Contains the subproblem generated by converting an fixed constraint 
+    \sum_{i}a_i x_i = b
+    into an elastic constraint
+    @param constraint: The LpConstraint that the elastic constraint is based on
+    @param penalty: penalty applied for violation (+ve or -ve) of the constraints
+    @param proportionFreeBound: the proportional bound (+ve and -ve) on 
+        constraint violation that is free from penalty
+    @param proportionFreeBoundList: the proportional bound on 
+        constraint violation that is free from penalty, expressed as a list 
+        where [-ve, +ve]
+    """
+    def __init__(self, constraint, penalty = None,
+                                        proportionFreeBound = None,
+                                        proportionFreeBoundList = None):
+        subProblemName =  "%s_elastic_SubProblem" % constraint.name
+        LpProblem.__init__(self, subProblemName, LpMinimize)
+        self.constraint = constraint
+        self.constant = constraint.constant
+        self.RHS = - constraint.constant
+        self.objective = LpAffineExpression()
+        self += constraint, "_Constraint"
+        if proportionFreeBound:
+            proportionFreeBoundList = [proportionFreeBound, proportionFreeBound]  
+        if proportionFreeBoundList:
+            #add a costless variable  
+            name = subProblemName + "_free_bound"
+            lowbound = abs(constraint.constant * proportionFreeBoundList[0])
+            upbound = abs(constraint.constant * proportionFreeBoundList[1])
+            # Note the reversal of the upbound and lowbound due to the nature of the
+            # variable
+            freeVar = LpVariable(name, lowBound = -upbound, upBound = +lowbound)
+            constraint.addInPlace(freeVar)
+            self.freeVar = freeVar
+        if penalty:
+            #add two penalty variables  
+            name = subProblemName + "_pos_penalty_var"
+            upVar = LpVariable(name, lowBound = 0)
+            name = subProblemName + "_neg_penalty_var"
+            lowVar = LpVariable(name, upBound = 0)
+            constraint.addInPlace(lowVar + upVar)
+            self.objective += penalty*upVar - penalty*lowVar      
+            self.upVar = upVar
+            self.lowVar = lowVar
 
+    def _findValue(self, attrib):
+        """
+        safe way to get the value of a variable that may not exist
+        """
+        var = getattr(self, attrib, 0)
+        if var:
+            return value(var)
+        else:
+            return 0            
 
+    def isViolated(self):
+        """
+        returns true if the penalty variables are non-zero
+        """
+        upVar = _findValue(self,"upVar")
+        lowVar = _findValue(self,"lowVar")
+        freeVar = _findValue(self,"freeVar")
+        result = abs(upVar + lowVar) >= EPS
+        if result:
+            logging.debug("isViolated %s, upVar %s, lowVar %s, freeVar %s result %s"%(
+                        self.name, upVar, lowVar, freeVar, result))
+            logging.debug("isViolated value lhs %s constant %s"%(
+                         self.findLHSValue(), self.RHS))
+        return result
+
+    def findDifferenceFromRHS(self):
+        """
+        This is the amount the actual value varies from the RHS
+        """
+        return self.findLHSValue() - self.RHS
+
+    def findLHSValue(self):
+        """
+        for elastic constraints finds the LHS value of the constraint without 
+        the free variable and or penalty variable assumes the constant is on the
+        rhs
+        """
+        upVar = self._findValue("upVar")
+        lowVar = self._findValue("lowVar")
+        freeVar = self._findValue("freeVar")
+        return self.constraint.value() - self.constant - \
+                upVar - lowVar - freeVar
+    
+    def alterName(self, name):
+        """
+        Alters the name of anonymous parts of the problem
+        
+        """
+        self.name = "%s_elastic_SubProblem" % name
+        if hasattr(self, 'freeVar'):
+            self.freeVar.name = self.name + "_free_bound"
+        if hasattr(self, 'upVar'):
+            self.upVar.name = self.name + "_pos_penalty_var"
+        if hasattr(self, 'lowVar'):
+            self.lowVar.name = self.name + "_neg_penalty_var"
+        
+
+class FractionElasticSubProblem(FixedElasticSubProblem):
+    """
+    Contains the subproblem generated by converting an Fraction constraint 
+    numerator/(numerator+complement) = b
+    into an elastic constraint
+    @param constraint: The LpFractionConstraint that the elastic constraint is 
+    based on
+    @param penalty: penalty applied for violation (+ve or -ve) of the constraints
+    @param proportionFreeBound: the proportional bound (+ve and -ve) on 
+        constraint violation that is free from penalty
+    @param proportionFreeBoundList: the proportional bound on 
+        constraint violation that is free from penalty, expressed as a list 
+        where [-ve, +ve]
+    """
+    def __init__(self, constraint, penalty = None,
+                                        proportionFreeBound = None,
+                                        proportionFreeBoundList = None):
+        subProblemName = "%s_elastic_SubProblem" % constraint.name
+        self.numerator = constraint.numerator
+        self.complement = constraint.complement
+        self.denominator = constraint.denominator
+        self.RHS = constraint.RHS
+        LpProblem.__init__(self, subProblemName, LpMinimize)
+        if proportionFreeBound:
+            proportionFreeBoundList = [proportionFreeBound, proportionFreeBound]  
+        if proportionFreeBoundList:
+            upProportionFreeBound, lowProportionFreeBound = \
+                    proportionFreeBoundList
+        else:
+            upProportionFreeBound, lowProportionFreeBound = (0, 0)
+        #create an objective
+        self += LpAffineExpression()
+        #There are three cases if the constraint.sense is ==, <=, >=
+        if constraint.sense in [LpConstraintEQ, LpConstraintLE]:
+            #create a constraint the sets the upper bound of target
+            upTarget = constraint.RHS + upProportionFreeBound
+            self.upConstraint = LpFractionConstraint(self.numerator, 
+                                    self.complement, 
+                                    LpConstraintLE, upTarget)
+            if penalty is not None:
+                self.lowVar = LpVariable(subProblemName + 
+                                             '_neg_penalty_var',
+                                             upBound = 0)
+                self.objective += -1* penalty * self.lowVar
+                self.upConstraint += self.lowVar
+            self += self.upConstraint, '_upper_constraint'
+        if constraint.sense in [LpConstraintEQ, LpConstraintGE]:
+            #create a constraint the sets the lower bound of target
+            lowTarget = constraint.RHS - lowProportionFreeBound
+            self.lowConstraint = LpFractionConstraint(self.numerator, 
+                                                 self.complement, 
+                                                LpConstraintGE, lowTarget)
+            if penalty is not None:
+                self.upVar = LpVariable(subProblemName + 
+                                             '_pos_penalty_var',
+                                             lowBound = 0)
+                self.objective += penalty * self.upVar
+                self.lowConstraint += self.upVar
+            self += self.lowConstraint, '_lower_constraint'
+        
+    def findLHSValue(self):
+        """
+        for elastic constraints finds the LHS value of the constraint without 
+        the free variable and or penalty variable assumes the constant is on the
+        rhs
+        """
+        # uses code from LpFractionConstraint
+        if abs(value(self.denominator))>= EPS:
+            return value(self.numerator)/value(self.denominator)
+        else:
+            if abs(value(self.numerator))<= EPS:
+                #zero divided by zero will return 1
+                return 1.0
+            else:
+                raise ZeroDivisionError
+    
+        
 class LpVariableDict(dict):
     """An LP variable generator"""
     def __init__(self, name, data = {}, lowBound = None, upBound = None, cat = 0):
@@ -1696,7 +1966,7 @@ def pulpTestCheck(prob, solver, okstatus, sol = {},
                    reducedcosts = None,
                    duals = None,
                    slacks = None,
-                   eps = EPS,
+                   eps = 10**-3,
                    status = None):
     
     if status is None:
@@ -1705,21 +1975,21 @@ def pulpTestCheck(prob, solver, okstatus, sol = {},
         prob.writeLP("debug.lp")
         prob.writeMPS("debug.mps")
         print "Failure: status ==", status, "not in", okstatus
-        raise PulpError, "Tests failed for solver ", solver
+        raise PulpError, "Tests failed for solver %s"%solver
     if sol:
         for v,x in sol.iteritems():
             if abs(v.varValue - x) > eps:
                 prob.writeLP("debug.lp")
                 prob.writeMPS("debug.mps")
                 print "Test failed: var", v, "==", v.varValue, "!=", x
-                raise PulpError, "Tests failed for solver ", solver
+                raise PulpError, "Tests failed for solver %s"%solver
     if reducedcosts:
         for v,dj in reducedcosts.iteritems():
             if abs(v.dj - dj) > eps:
                 prob.writeLP("debug.lp")
                 prob.writeMPS("debug.mps")
                 print "Test failed: var.dj", v, "==", v.dj, "!=", dj
-                raise PulpError, "Tests failed for solver ", solver
+                raise PulpError, "Tests failed for solver %s"%solver
     if duals:
         for cname,p in duals.iteritems():
             c = prob.constraints[cname]
@@ -1727,7 +1997,7 @@ def pulpTestCheck(prob, solver, okstatus, sol = {},
                 prob.writeLP("debug.lp")
                 prob.writeMPS("debug.mps")
                 print "Test failed: constraint.pi", cname , "==", c.pi, "!=", p
-                raise PulpError, "Tests failed for solver ", solver
+                raise PulpError, "Tests failed for solver %s"%solver
     if slacks:
         for cname,slack in slacks.iteritems():
             c = prob.constraints[cname]
@@ -1736,11 +2006,11 @@ def pulpTestCheck(prob, solver, okstatus, sol = {},
                 prob.writeMPS("debug.mps")
                 print ("Test failed: constraint.slack", cname , "==",
                         c.slack, "!=", slack)
-                raise PulpError, "Tests failed for solver ", solver
+                raise PulpError, "Tests failed for solver %s"%solver
         
-def pulpTest01(solver):
+def pulpTest010(solver):
     # Continuous
-    prob = LpProblem("test1", LpMinimize)
+    prob = LpProblem("test010", LpMinimize)
     x = LpVariable("x", 0, 4)
     y = LpVariable("y", -1, 1)
     z = LpVariable("z", 0)
@@ -1753,9 +2023,9 @@ def pulpTest01(solver):
     print "\t Testing continuous LP solution"
     pulpTestCheck(prob, solver, [LpStatusOptimal], {x:4, y:-1, z:6, w:0})
 
-def pulpTest11(solver):
+def pulpTest011(solver):
     # Continuous Maximisation
-    prob = LpProblem("test1", LpMaximize)
+    prob = LpProblem("test011", LpMaximize)
     x = LpVariable("x", 0, 4)
     y = LpVariable("y", -1, 1)
     z = LpVariable("z", 0)
@@ -1768,9 +2038,31 @@ def pulpTest11(solver):
     print "\t Testing maximize continuous LP solution"
     pulpTestCheck(prob, solver, [LpStatusOptimal], {x:4, y:1, z:8, w:0})
 
-def pulpTest02(solver):
+def pulpTest012(solver):
+    # Unbounded
+    prob = LpProblem("test012", LpMaximize)
+    x = LpVariable("x", 0, 4)
+    y = LpVariable("y", -1, 1)
+    z = LpVariable("z", 0)
+    w = LpVariable("w", 0)
+    prob += x + 4*y + 9*z + w, "obj"
+    prob += x+y <= 5, "c1"
+    prob += x+z >= 10, "c2"
+    prob += -y+z == 7, "c3"
+    prob += w >= 0, "c4"
+    print "\t Testing unbounded continuous LP solution"
+    if solver.__class__ in [COINMP_DLL, GUROBI]:
+            # COINMP_DLL Does not report unbounded problems, correctly
+             pulpTestCheck(prob, solver, [LpStatusInfeasible])
+    elif solver.__class__ is GLPK_CMD:
+        # GLPK_CMD Does not report unbounded problems, correctly
+         pulpTestCheck(prob, solver, [LpStatusUndefined])
+    else:
+        pulpTestCheck(prob, solver, [LpStatusUnbounded])
+
+def pulpTest020(solver):
     # MIP
-    prob = LpProblem("test2", LpMinimize)
+    prob = LpProblem("test020", LpMinimize)
     x = LpVariable("x", 0, 4)
     y = LpVariable("y", -1, 1)
     z = LpVariable("z", 0, None, LpInteger)
@@ -1778,16 +2070,12 @@ def pulpTest02(solver):
     prob += x+y <= 5, "c1"
     prob += x+z >= 10, "c2"
     prob += -y+z == 7.5, "c3"
-    if solver.__class__ in [COIN_CMD]:
-        # COIN_CMD always return LpStatusUndefined for MIP problems
-        pulpTestCheck(prob, solver, [LpStatusUndefined], {x:3, y:-0.5, z:7})
-    else:
-        print "\t Testing MIP solution"
-        pulpTestCheck(prob, solver, [LpStatusOptimal], {x:3, y:-0.5, z:7})
+    print "\t Testing MIP solution"
+    pulpTestCheck(prob, solver, [LpStatusOptimal], {x:3, y:-0.5, z:7})
 
-def pulpTest03(solver):
+def pulpTest030(solver):
     # relaxed MIP
-    prob = LpProblem("test3", LpMinimize)
+    prob = LpProblem("test030", LpMinimize)
     x = LpVariable("x", 0, 4)
     y = LpVariable("y", -1, 1)
     z = LpVariable("z", 0, None, LpInteger)
@@ -1800,28 +2088,22 @@ def pulpTest03(solver):
     pulpTestCheck(prob, solver, [LpStatusOptimal], {x:3.5, y:-1, z:6.5})
 
 
-def pulpTest04(solver):
+def pulpTest040(solver):
     # Feasibility only
-    prob = LpProblem("test4", LpMinimize)
+    prob = LpProblem("test040", LpMinimize)
     x = LpVariable("x", 0, 4)
     y = LpVariable("y", -1, 1)
     z = LpVariable("z", 0, None, LpInteger)
     prob += x+y <= 5, "c1"
     prob += x+z >= 10, "c2"
     prob += -y+z == 7.5, "c3"
-    if solver.__class__ in [COIN_CMD]:
-        # COIN_CMD always return LpStatusUndefined
-        pulpTestCheck(prob, solver, [LpStatusUndefined])
-        if x.varValue is None or x.varValue is None or x.varValue is None:
-            raise "Tests failed for solver ", solver
-    else:
-        print "\t Testing feasibility problem (no objective)"
-        pulpTestCheck(prob, solver, [LpStatusOptimal])
+    print "\t Testing feasibility problem (no objective)"
+    pulpTestCheck(prob, solver, [LpStatusOptimal])
 
 
-def pulpTest05(solver):
+def pulpTest050(solver):
     # Infeasible
-    prob = LpProblem("test5", LpMinimize)
+    prob = LpProblem("test050", LpMinimize)
     x = LpVariable("x", 0, 4)
     y = LpVariable("y", -1, 1)
     z = LpVariable("z", 0, 10)
@@ -1839,23 +2121,20 @@ def pulpTest05(solver):
         print "\t Testing an infeasible problem"
         pulpTestCheck(prob, solver, [LpStatusInfeasible])
 
-def pulpTest06(solver):
+def pulpTest060(solver):
     # Integer Infeasible
-    prob = LpProblem("test6", LpMinimize)
+    prob = LpProblem("test060", LpMinimize)
     x = LpVariable("x", 0, 4, LpInteger)
     y = LpVariable("y", -1, 1, LpInteger)
     z = LpVariable("z", 0, 10, LpInteger)
     prob += x+y <= 5.2, "c1"
     prob += x+z >= 10.3, "c2"
     prob += -y+z == 7.4, "c3"
-    if solver.__class__ in [ COIN_CMD]:
-        # COIN_CMD integer return code is always LpStatusUndefined
-        pulpTestCheck(prob, solver, [LpStatusUndefined])
-    elif solver.__class__ in [ GLPK_CMD]:
+    if solver.__class__ in [ GLPK_CMD]:
         # GLPK_CMD returns InfeasibleOrUnbounded
         print "\t Testing an integer infeasible problem"
         pulpTestCheck(prob, solver, [LpStatusInfeasible, LpStatusUndefined])
-    elif solver.__class__ in [COINMP_DLL]:
+    elif solver.__class__ in [COINMP_DLL, COIN_CMD]:
         #Currently there is an error in COINMP for problems where
         #presolve elimiates too many variables
         print "\t Testing an integer infeasible problem (Error to be fixed)"
@@ -1864,9 +2143,9 @@ def pulpTest06(solver):
         print "\t Testing an integer infeasible problem"
         pulpTestCheck(prob, solver, [LpStatusInfeasible])
 
-def pulpTest07(solver):
+def pulpTest070(solver):
     #Column Based modelling of PulpTest1
-    prob = LpProblem("test7", LpMinimize)
+    prob = LpProblem("test070", LpMinimize)
     obj = LpConstraintVar("obj")
     # constraints
     a = LpConstraintVar("C1", LpConstraintLE, 5)
@@ -1886,7 +2165,7 @@ def pulpTest07(solver):
 
 def pulpTest075(solver):
     #Column Based modelling of PulpTest1 with empty constraints
-    prob = LpProblem("test7", LpMinimize)
+    prob = LpProblem("test075", LpMinimize)
     obj = LpConstraintVar("obj")
     # constraints
     a = LpConstraintVar("C1", LpConstraintLE, 5)
@@ -1905,11 +2184,11 @@ def pulpTest075(solver):
         print "\t Testing column based modelling with empty constraints"
         pulpTestCheck(prob, solver, [LpStatusOptimal], {x:4, y:-1, z:6})
 
-def pulpTest08(solver):
+def pulpTest080(solver):
     """
     Test the reporting of dual variables slacks and reduced costs
     """
-    prob = LpProblem("test8", LpMinimize)
+    prob = LpProblem("test080", LpMinimize)
     x = LpVariable("x", 0, 5)
     y = LpVariable("y", -1, 1)
     z = LpVariable("z", 0)
@@ -1930,9 +2209,9 @@ def pulpTest08(solver):
                   duals = {"c1":0, "c2":1, "c3":8}, 
                   slacks = {"c1":2, "c2":0, "c3":0})
 
-def pulpTest09(solver):
+def pulpTest090(solver):
     #Column Based modelling of PulpTest1 with a resolve
-    prob = LpProblem("test9", LpMinimize)
+    prob = LpProblem("test090", LpMinimize)
     obj = LpConstraintVar("obj")
     # constraints
     a = LpConstraintVar("C1", LpConstraintLE, 5)
@@ -1956,12 +2235,12 @@ def pulpTest09(solver):
         #over ridden if it is not implemented
         #pulpTestCheck(prob, solver, [LpStatusOptimal], {x:4, y:-1, z:6})
 
-def pulpTest10(solver):
+def pulpTest100(solver):
     """
     Test the ability to sequentially solve a problem
     """
     # set up a cubic feasible region
-    prob = LpProblem("test10", LpMinimize)
+    prob = LpProblem("test100", LpMinimize)
     x = LpVariable("x", 0, 1)
     y = LpVariable("y", 0, 1)
     z = LpVariable("z", 0, 1)
@@ -1977,18 +2256,132 @@ def pulpTest10(solver):
                   sol = {x:0, y:1}, 
                   status = status)
 
+def pulpTest110(solver):
+    """
+    Test the ability to use fractional constraints
+    """
+    prob = LpProblem("test110", LpMinimize)
+    x = LpVariable("x", 0, 4)
+    y = LpVariable("y", -1, 1)
+    z = LpVariable("z", 0)
+    w = LpVariable("w", 0)
+    prob += x + 4*y + 9*z, "obj"
+    prob += x+y <= 5, "c1"
+    prob += x+z >= 10, "c2"
+    prob += -y+z == 7, "c3"
+    prob += w >= 0, "c4"
+    prob += LpFractionConstraint(x, z, LpConstraintEQ, 0.5, name = 'c5')
+    print "\t Testing fractional constraints"
+    pulpTestCheck(prob, solver, [LpStatusOptimal], 
+                    {x:10/3.0, y:-1/3.0, z:20/3.0, w:0})
+
+def pulpTest120(solver):
+    """
+    Test the ability to use Elastic constraints
+    """
+    prob = LpProblem("test120", LpMinimize)
+    x = LpVariable("x", 0, 4)
+    y = LpVariable("y", -1, 1)
+    z = LpVariable("z", 0)
+    w = LpVariable("w")
+    prob += x + 4*y + 9*z + w, "obj"
+    prob += x+y <= 5, "c1"
+    prob += x+z >= 10, "c2"
+    prob += -y+z == 7, "c3"
+    prob.extend((w >= -1).makeElasticSubProblem())
+    print "\t Testing elastic constraints (no change)"
+    pulpTestCheck(prob, solver, [LpStatusOptimal], 
+                    {x:4, y:-1, z:6, w:-1})
+
+def pulpTest121(solver):
+    """
+    Test the ability to use Elastic constraints
+    """
+    prob = LpProblem("test121", LpMinimize)
+    x = LpVariable("x", 0, 4)
+    y = LpVariable("y", -1, 1)
+    z = LpVariable("z", 0)
+    w = LpVariable("w")
+    prob += x + 4*y + 9*z + w, "obj"
+    prob += x+y <= 5, "c1"
+    prob += x+z >= 10, "c2"
+    prob += -y+z == 7, "c3"
+    prob.extend((w >= -1).makeElasticSubProblem(proportionFreeBound = 0.1))
+    print "\t Testing elastic constraints (freebound)"
+    pulpTestCheck(prob, solver, [LpStatusOptimal], 
+                    {x:4, y:-1, z:6, w:-1.1})
+
+def pulpTest122(solver):
+    """
+    Test the ability to use Elastic constraints (penalty unchanged)
+    """
+    prob = LpProblem("test122", LpMinimize)
+    x = LpVariable("x", 0, 4)
+    y = LpVariable("y", -1, 1)
+    z = LpVariable("z", 0)
+    w = LpVariable("w")
+    prob += x + 4*y + 9*z + w, "obj"
+    prob += x+y <= 5, "c1"
+    prob += x+z >= 10, "c2"
+    prob += -y+z == 7, "c3"
+    prob.extend((w >= -1).makeElasticSubProblem(penalty = 1.1))
+    print "\t Testing elastic constraints (penalty unchanged)"
+    prob.writeLP('debug.lp')
+    pulpTestCheck(prob, solver, [LpStatusOptimal], 
+                    {x:4, y:-1, z:6, w:-1.0})
+
+def pulpTest123(solver):
+    """
+    Test the ability to use Elastic constraints (penalty unbounded)
+    """
+    prob = LpProblem("test123", LpMinimize)
+    x = LpVariable("x", 0, 4)
+    y = LpVariable("y", -1, 1)
+    z = LpVariable("z", 0)
+    w = LpVariable("w")
+    prob += x + 4*y + 9*z + w, "obj"
+    prob += x+y <= 5, "c1"
+    prob += x+z >= 10, "c2"
+    prob += -y+z == 7, "c3"
+    prob.extend((w >= -1).makeElasticSubProblem(penalty = 0.9))
+    print "\t Testing elastic constraints (penalty unbounded)"
+    prob.writeLP('debug.lp')
+    if solver.__class__ in [COINMP_DLL, GUROBI]:
+        # COINMP_DLL Does not report unbounded problems, correctly
+         pulpTestCheck(prob, solver, [LpStatusInfeasible])
+    elif solver.__class__ is GLPK_CMD:
+        # GLPK_CMD Does not report unbounded problems, correctly
+         pulpTestCheck(prob, solver, [LpStatusUndefined])
+    else:
+        pulpTestCheck(prob, solver, [LpStatusUnbounded])
+
+
 def pulpTestSolver(solver):
-    tests = [pulpTest01, pulpTest11, pulpTest02, pulpTest03, pulpTest04,
-        pulpTest05, pulpTest06, pulpTest07, pulpTest075,
-        pulpTest08, pulpTest09, pulpTest10]
+    tests = [pulpTest010, pulpTest011, pulpTest012, 
+            pulpTest020, 
+            pulpTest030, 
+            pulpTest040,
+            pulpTest050, 
+            pulpTest060, 
+            pulpTest070, pulpTest075,
+            pulpTest080, 
+            pulpTest090, 
+            pulpTest100, 
+            pulpTest110, 
+            pulpTest120, pulpTest121, pulpTest122, pulpTest123
+            ]
     for t in tests:
         t(solver(msg=0))
 
 def pulpTestAll():
-    solvers = [CPLEX_DLL, CPLEX_CMD,
-         COIN_CMD, COINMP_DLL,
-         GLPK_CMD,
-        XPRESS, GUROBI]
+    solvers = [#CPLEX_DLL, 
+               #CPLEX_CMD,
+               COIN_CMD, 
+               COINMP_DLL,
+               GLPK_CMD,
+               XPRESS, 
+               GUROBI
+               ]
 
     for s in solvers:
         if s().available():
