@@ -47,6 +47,7 @@ class PulpSolverError(Exception):
     """
     Pulp Solver-related exceptions
     """
+    pass
 
 #import configuration information
 def initialize(file = None):
@@ -97,13 +98,13 @@ class LpSolver:
         """Solve a well formulated lp problem"""
         raise NotImplementedError
 
-    def actualResolve(self,lp):
+    def actualResolve(self,lp, **kwargs):
         """
         uses existing problem information and solves the problem
         If it is not implelemented in the solver
         just solve again
         """
-        self.actualSolve(lp)
+        self.actualSolve(lp, **kwargs)
 
     def copy(self):
         """Make a copy of self"""
@@ -341,8 +342,9 @@ class GLPK_CMD(LpSolver_CMD):
             "UNDEFINED":LpStatusUndefined,
             "INTEGER EMPTY":LpStatusInfeasible
             }
+        #print "statusString ",statusString
         if statusString not in glpkStatus:
-            raise ValueError, "Unknow status returned by GLPK"
+            raise PulpSolverError, "Unknown status returned by GLPK"
         status = glpkStatus[statusString]
         isInteger = statusString in ["INTEGER OPTIMAL","INTEGER UNDEFINED"]
         values = {}
@@ -431,7 +433,10 @@ class CPLEX_CMD(LpSolver_CMD):
 
     def readsol(self,filename):
         """Read a CPLEX solution file"""
-        import xml.etree.ElementTree as et
+        try:
+            import xml.etree.ElementTree as et
+        except ImportError:
+            import elementtree.ElementTree as et
         solutionXML = et.parse(filename).getroot()
         solutionheader = solutionXML.find("header")
         statusString = solutionheader.get("solutionStatusString")
@@ -439,7 +444,7 @@ class CPLEX_CMD(LpSolver_CMD):
             "optimal":LpStatusOptimal,
             }
         if statusString not in cplexStatus:
-            raise ValueError, "Unknown status returned by CPLEX: "+statusString
+            raise PulpSolverError, "Unknown status returned by CPLEX: "+statusString
         status = cplexStatus[statusString]
         
         shadowPrices = {}
@@ -467,7 +472,6 @@ class CPLEX_CMD(LpSolver_CMD):
 
 try:
     import ctypes
-        
     class CPLEX_DLL(LpSolver):
         """
         The CPLEX LP/MIP solver (via a Dynamic library DLL - windows or SO - Linux)
@@ -477,16 +481,29 @@ try:
         For api functions that have not been wrapped in this solver please use 
         the ctypes library interface to the cplex api in CPLEX_DLL.lib
         """
-        EPGAP = 2009
-        CPX_PARAM_MEMORYEMPHASIS = 1082 # from Cplex 11.0 manual
         if os.name in ['nt','dos']:
             lib = ctypes.windll.LoadLibrary(cplex_dll_path)
         else:
             lib = ctypes.cdll.LoadLibrary(cplex_dll_path)
+        #parameters manually found in solver manual
+        CPX_PARAM_EPGAP = 2009
+        CPX_PARAM_MEMORYEMPHASIS = 1082 # from Cplex 11.0 manual
+        CPX_PARAM_TILIM = 1039
+        #argtypes for CPLEX functions
+        lib.CPXsetintparam.argtypes = [ctypes.c_void_p, 
+                         ctypes.c_int, ctypes.c_int]
+        lib.CPXsetdblparam.argtypes = [ctypes.c_void_p, ctypes.c_int, 
+                                                ctypes.c_double]
+        lib.CPXfopen.argtypes = [ctypes.c_char_p, 
+                                      ctypes.c_char_p]
+        lib.CPXfopen.restype = ctypes.c_void_p
+        lib.CPXsetlogfile.argtypes = [ctypes.c_void_p, 
+                                      ctypes.c_void_p] 
+
         def __init__(self, 
                     mip = True, 
                     msg = True, 
-                    timeLimit = -1,
+                    timeLimit = None,
                     epgap = None, 
                     logfilename = None, 
                     emphasizeMemory = False):
@@ -506,6 +523,8 @@ try:
             self.setMemoryEmphasis(emphasizeMemory)
             if epgap is not None:
                 self.changeEpgap(epgap)
+            if timeLimit is not None:
+                self.changeTimeLimit(timeLimit)
             if logfilename is not None:
                 self.setlogfile(logfilename)
             else:
@@ -515,29 +534,28 @@ try:
             """
             sets the logfile for cplex output
             """
-            CPLEX_DLL.lib.CPXfopen.argtypes = [ctypes.c_char_p, 
-                                          ctypes.c_char_p]
-            CPLEX_DLL.lib.CPXfopen.restype = ctypes.c_void_p
             self.logfilep = CPLEX_DLL.lib.CPXfopen(filename, "w")
-            CPLEX_DLL.lib.CPXsetlogfile.argtypes = [ctypes.c_void_p, 
-                                          ctypes.c_void_p] 
             CPLEX_DLL.lib.CPXsetlogfile(self.env, self.logfilep)
         
         def changeEpgap(self, epgap = 10**-4):
             """
             Change cplex solver integer bound gap tolerence
             """
-            CPLEX_DLL.lib.CPXsetdblparam.argtypes = [ctypes.c_void_p, ctypes.c_int, 
-                                                ctypes.c_double]
-            CPLEX_DLL.lib.CPXsetdblparam(self.env,CPLEX_DLL.EPGAP,epgap)
+            CPLEX_DLL.lib.CPXsetdblparam(self.env,CPLEX_DLL.CPX_PARAM_EPGAP,
+                                            epgap)
+
+        def setTimeLimit(self, timeLimit = 0.0):
+            """
+            Make cplex limit the time it takes --added CBM 8/28/09
+            """
+            CPLEX_DLL.lib.CPXsetdblparam(self.env,CPLEX_DLL.CPX_PARAM_TILIM,
+                                                float(timeLimit))
 
         def setMemoryEmphasis(self, yesOrNo = False):
             """
             Make cplex try to conserve memory at the expense of
             performance.
             """
-            CPLEX_DLL.lib.CPXsetintparam.argtypes = [ctypes.c_void_p, 
-                             ctypes.c_int, ctypes.c_int]
             CPLEX_DLL.lib.CPXsetintparam(self.env,
                             CPLEX_DLL.CPX_PARAM_MEMORYEMPHASIS,yesOrNo)
             
@@ -557,27 +575,26 @@ try:
                 status = CPLEX_DLL.lib.CPXgetobjval(self.env, self.hprob, 
                                                     byref(objectiveValue))
                 if status != 0 and status != 1217: #no solution exists
-                    raise RuntimeError, ("Error in CPXgetobjval status=" 
+                    raise PulpSolverError, ("Error in CPXgetobjval status=" 
                                           + str(status))
                 
                 status = CPLEX_DLL.lib.CPXgetx(self.env, self.hprob, 
                                                 byref(x), 0, numcols - 1)
                 if status != 0 and status != 1217:
-                    raise RuntimeError, "Error in CPXgetx status=" + str(status)                
+                    raise PulpSolverError, "Error in CPXgetx status=" + str(status)                
             else:
                 status = CPLEX_DLL.lib.CPXsolution(self.env, self.hprob, 
                                               byref(solutionStatus), 
                                               byref(objectiveValue), 
                                               byref(x), byref(pi), 
                                               byref(slack), byref(dj))
-            CplexLpStatus = collections.defaultdict(lambda : LpStatusUndefined)
             # 102 is the cplex return status for
             # integer optimal within tolerance
             # and is useful for breaking symmetry.
-            CplexLpStatus.update({1: LpStatusOptimal, 3: LpStatusInfeasible, 
+            CplexLpStatus = {1: LpStatusOptimal, 3: LpStatusInfeasible, 
                                   2: LpStatusUnbounded, 0: LpStatusNotSolved,
                                   101: LpStatusOptimal, 102: LpStatusOptimal,
-                                  103: LpStatusInfeasible})
+                                  103: LpStatusInfeasible}
             #populate pulp solution values
             variablevalues = {}
             variabledjvalues = {}
@@ -600,7 +617,8 @@ try:
             lp.resolveOK = True
             for var in lp.variables():
                 var.isModified = False
-            lp.status = CplexLpStatus[solutionStatus.value]
+            lp.status = CplexLpStatus.get(solutionStatus.value,
+                                          LpStatusUndefined)
             return lp.status
 
         def __del__(self):
@@ -619,7 +637,7 @@ try:
             status = ctypes.c_int()
             self.env = CPLEX_DLL.lib.CPXopenCPLEX(ctypes.byref(status))
             if not(status.value == 0):
-                raise RuntimeError, ("CPLEX library failed on " + 
+                raise PulpSolverError, ("CPLEX library failed on " + 
                                     "CPXopenCPLEX status=" + str(status))
             
 
@@ -628,7 +646,7 @@ try:
             if getattr(self,"env",False):
                 status=CPLEX_DLL.lib.CPXcloseCPLEX(self.env)
             else:
-                raise RuntimeError, "No CPLEX enviroment to close"
+                raise PulpSolverError, "No CPLEX enviroment to close"
 
         def callSolver(self, isMIP):
             """Solves the problem with cplex
@@ -638,12 +656,12 @@ try:
             if isMIP and self.mip:
                 status= CPLEX_DLL.lib.CPXmipopt(self.env, self.hprob)
                 if status != 0:
-                    raise RuntimeError, ("Error in CPXmipopt status=" 
+                    raise PulpSolverError, ("Error in CPXmipopt status=" 
                                         + str(status))
             else:
                 status = CPLEX_DLL.lib.CPXlpopt(self.env, self.hprob)
                 if status != 0:
-                    raise RuntimeError, ("Error in CPXlpopt status=" 
+                    raise PulpSolverError, ("Error in CPXlpopt status=" 
                                             + str(status))
             self.cplexTime += clock()
 
@@ -655,7 +673,7 @@ try:
             self.hprob = CPLEX_DLL.lib.CPXcreateprob(self.env, 
                                                     byref(status), lp.name)
             if status.value != 0:
-                raise RuntimeError, ("Error in CPXcreateprob status=" 
+                raise PulpSolverError, ("Error in CPXcreateprob status=" 
                                     + str(status)) 
             (numcols, numrows, numels, rangeCount, 
                 objSense, obj, objconst, 
@@ -667,14 +685,14 @@ try:
                                  objSense, obj, rhs, rowSense, matbeg, matcnt,
                                  matind, matval, lb, ub, None, colname, rowname)
             if status.value != 0:
-                raise RuntimeError, ("Error in CPXcopylpwnames status=" + 
+                raise PulpSolverError, ("Error in CPXcopylpwnames status=" + 
                                         str(status)) 
             if lp.isMIP() and self.mip:
                 status.value = CPLEX_DLL.lib.CPXcopyctype(self.env, 
                                                           self.hprob, 
                                                           xctype)
             if status.value != 0:
-                raise RuntimeError, ("Error in CPXcopyctype status=" + 
+                raise PulpSolverError, ("Error in CPXcopyctype status=" + 
                                         str(status)) 
             #set the initial solution            
             self.callSolver(lp.isMIP())
@@ -807,7 +825,7 @@ except (ImportError,OSError):
             return False
         def actualSolve(self, lp):
             """Solve a well formulated lp problem"""
-            raise RuntimeError, "CPLEX_DLL: Not Available"
+            raise PulpSolverError, "CPLEX_DLL: Not Available"
     CPLEX = CPLEX_CMD
 
 
@@ -874,7 +892,7 @@ class XPRESS(LpSolver_CMD):
             "Optimal":LpStatusOptimal,
             }
         if statusString not in xpressStatus:
-            raise ValueError, "Unknow status returned by XPRESS: "+statusString
+            raise PulpSolverError, "Unknow status returned by XPRESS: "+statusString
         status = xpressStatus[statusString]
         values = {}
         while 1:
@@ -913,8 +931,9 @@ class COIN_CMD(LpSolver_CMD):
 
     def actualSolve(self, lp):
         """Solve a well formulated lp problem"""
-        if lp.isMIP() and self.mip: return self.solve_CBC(lp)
-        else: return self.solve_CLP(lp)
+        return self.solve_CBC(lp)
+        #if lp.isMIP() and self.mip: return self.solve_CBC(lp)
+        #else: return self.solve_CLP(lp)
 
     def available(self):
         """True if the solver is available"""
@@ -924,36 +943,30 @@ class COIN_CMD(LpSolver_CMD):
     def solve_CBC(self, lp):
         """Solve a MIP problem using CBC"""
         if not self.executable(self.path[1]):
-            raise "PuLP: cannot execute "+self.path[1]
+            raise PulpSolverError, "PuLP: cannot execute "+self.path[1]
         if not self.keepFiles:
             pid = os.getpid()
-            tmpLp = os.path.join(self.tmpDir, "%d-pulp.mps" % pid)
+            tmpLp = os.path.join(self.tmpDir, "%d-pulp.lp" % pid)
             tmpSol = os.path.join(self.tmpDir, "%d-pulp.sol" % pid)
         else:
-            tmpLp = lp.name+"-pulp.mps"
+            tmpLp = lp.name+"-pulp.lp"
             tmpSol = lp.name+"-pulp.sol"
-        vs, variablesNames, constraintsNames, objectiveName = lp.writeMPS(tmpLp, rename = 1)
-#        if not self.msg:
-#            cbc = os.popen(self.path[1]+" - > /dev/null 2> /dev/null","w")
-#        else:
-#            cbc = os.popen(self.path[1]+" -","w")
-        #TODO make this os agnostic 
-        cbc = os.popen(self.path[1]+" -","w")
+        lp.writeLP(tmpLp)
+        if not self.msg and os.name == 'posix':
+            cbc = os.popen(self.path[1]+" - > /dev/null 2> /dev/null","w")
+        else:
+            cbc = os.popen(self.path[1]+" -","w")
         cbc.write("import "+tmpLp+"\n")
         if self.presolve:
             cbc.write("presolve on\n")
         cbc.write("strong %d\n" % self.strong)
         if self.cuts:
             cbc.write("gomory on\n")
-            cbc.write("oddhole on\n")
+            #cbc.write("oddhole on\n")
             cbc.write("knapsack on\n")
             cbc.write("probing on\n")
         for option in self.options:
             cbc.write(option+"\n")
-        if lp.sense == LpMinimize:
-            cbc.write("min\n")
-        else:
-            cbc.write("max\n")
         if self.mip:
             cbc.write("branch\n")
         else:
@@ -961,10 +974,10 @@ class COIN_CMD(LpSolver_CMD):
         cbc.write("solution "+tmpSol+"\n")
         cbc.write("quit\n")
         if cbc.close() != None:
-            raise "PuLP: Error while trying to execute "+self.path[1]
+            raise PulpSolverError, "PuLP: Error while trying to execute "+self.path[1]
         if not os.path.exists(tmpSol):
-            raise "PuLP: Error while executing "+self.path[1]
-        lp.status, values = self.readsol_CBC(tmpSol, lp, vs)
+            raise PulpSolverError, "PuLP: Error while executing "+self.path[1]
+        lp.status, values = self.readsol_CBC(tmpSol, lp, lp.variables())
         lp.assignVarsVals(values)
         if not self.keepFiles:
             try: os.remove(tmpLp)
@@ -976,16 +989,16 @@ class COIN_CMD(LpSolver_CMD):
     def solve_CLP(self, lp):
         """Solve a problem using CLP"""
         if not self.executable(self.path[0]):
-            raise "PuLP: cannot execute "+self.path[0]
+            raise PulpSolverError, "PuLP: cannot execute "+self.path[0]
         if not self.keepFiles:
             pid = os.getpid()
-            tmpLp = os.path.join(self.tmpDir, "%d-pulp.mps" % pid)
+            tmpLp = os.path.join(self.tmpDir, "%d-pulp.lp" % pid)
             tmpSol = os.path.join(self.tmpDir, "%d-pulp.sol" % pid)
         else:
-            tmpLp = lp.name+"-pulp.mps"
+            tmpLp = lp.name+"-pulp.lp"
             tmpSol = lp.name+"-pulp.sol"
-        vs, variablesNames, constraintsNames, objectiveName = lp.writeMPS(tmpLp, rename = 1)
-        if not self.msg:
+        lp.writeLP(tmpLp)
+        if not self.msg and os.name == 'posix':
             clp = os.popen(self.path[0]+" - > /dev/null 2> /dev/null","w")
         else:
             clp = os.popen(self.path[0]+" -","w")
@@ -994,10 +1007,6 @@ class COIN_CMD(LpSolver_CMD):
             clp.write("presolve on\n")
         for option in self.options:
             clp.write(option+"\n")
-        if lp.sense == LpMinimize:
-            clp.write("min\n")
-        else:
-            clp.write("max\n")
         if self.dual:
             clp.write("dualS\n")
         else:
@@ -1005,10 +1014,10 @@ class COIN_CMD(LpSolver_CMD):
         clp.write("solution "+tmpSol+"\n")
         clp.write("quit\n")
         if clp.close() != None:
-            raise "PuLP: Error while trying to execute "+self.path[0]
+            raise PulpSolverError, "PuLP: Error while trying to execute "+self.path[0]
         if not os.path.exists(tmpSol):
-            raise "PuLP: Error while executing "+self.path[0]
-        lp.status, values = self.readsol_CLP(tmpSol, lp, vs, variablesNames, constraintsNames, objectiveName)
+            raise PulpSolverError, "PuLP: Error while executing "+self.path[0]
+        lp.status, values = self.readsol_CLP(tmpSol, lp, lp.variables())
         lp.assignVarsVals(values)
         if not self.keepFiles:
             try: os.remove(tmpLp)
@@ -1017,39 +1026,46 @@ class COIN_CMD(LpSolver_CMD):
             except: pass
         return lp.status
 
-    def readsol_CLP(self,filename, lp, vs, variablesNames, constraintsNames, objectiveName):
+    def readsol_CLP(self, filename, lp, vs):
         """Read a CLP solution file"""
         values = {}
-
-        reverseVn = {}
-        for k,n in variablesNames.iteritems():
-            reverseVn[n] = k
-
         for v in vs:
             values[v.name] = 0.0
 
+        clpStatus = {'optimal': LpStatusOptimal,
+                     'infeasible': LpStatusInfeasible,
+                     'unbounded': LpStatusUnbounded}
         status = LpStatusOptimal # status is very approximate
         f = file(filename)
+        #get the status line
+        statusstr = f.next().strip()
+        #print "CLP status ", statusstr
+        status = clpStatus.get(statusstr, LpStatusUndefined)
         for l in f:
             if len(l)<=2: break
-            if l[:2] == "**":
-                status = LpStatusInfeasible
-                l = l[2:]
             l = l.split()
             vn = l[1]
-            if vn in reverseVn:
-                values[reverseVn[vn]] = float(l[2])
+            if vn in values:
+                values[vn] = float(l[2])
         return status, values
 
     def readsol_CBC(self,filename, lp, vs):
         """Read a CBC solution file"""
-        f = file(filename)
-        for i in range(len(lp.constraints)): f.readline()
         values = {}
         for v in vs:
-            l = f.readline().split()
-            values[v.name] = float(l[1])
-        status = LpStatusUndefined # No status info
+            values[v.name] = 0.0
+        cbcStatus = {'Optimal':LpStatusOptimal, 
+                    'Infeasible':LpStatusInfeasible,
+                    'Unbounded':LpStatusUnbounded}
+        f = file(filename)
+        statusstr = f.readline().split()[0]
+        status = cbcStatus.get(statusstr, LpStatusUndefined)
+        for l in f:
+            if len(l)<=2: break
+            l = l.split()
+            vn = l[1]
+            if vn in values:
+                values[vn] = float(l[2])
         return status, values
 
 try:
@@ -1070,6 +1086,8 @@ try:
         CoinMP.CoinGetVersionStr.restype = ctypes.c_char_p
         CoinMP.CoinGetSolutionText.restype=ctypes.c_char_p
         CoinMP.CoinGetObjectValue.restype=ctypes.c_double
+        CoinMP.CoinGetMipBestBound.restype=ctypes.c_double
+
         def __init__(self, mip = 1, msg = 1, cuts = 1, presolve = 1, dual = 1,
             crash = 0, scale = 1, rounding = 1, integerPresolve = 1, strong = 5,
             timeLimit = None, epgap = None):
@@ -1165,7 +1183,7 @@ try:
                 savestdout = os.dup(1)
                 os.close(1)
                 if os.dup(tempfile.fileno()) != 1:
-                    raise RuntimeError, "couldn't redirect stdout - dup() error"
+                    raise PulpSolverError, "couldn't redirect stdout - dup() error"
             self.coinTime = -clock()
             CoinMP.CoinOptimizeProblem(hProb, 0);
             self.coinTime += clock()
@@ -1204,6 +1222,8 @@ try:
             variabledjvalues = {}
             constraintpivalues = {}
             constraintslackvalues = {}
+            if lp.isMIP() and self.mip:
+                lp.bestBound = CoinMP.CoinGetMipBestBound(hProb)
             for i in range(numVars):
                 variablevalues[self.n2v[i].name] = cActivity[i]
                 variabledjvalues[self.n2v[i].name] = cReducedCost[i]
@@ -1218,116 +1238,7 @@ try:
                        
             CoinMP.CoinFreeSolver()
             lp.status = CoinLpStatus[CoinMP.CoinGetSolutionStatus(hProb)]
-            return lp.status
-
-        def getCoinMPArrays(self,lp,coinDblMax):
-            """returns the arrays suitable to pass to a cdll CoinMP
-
-            Copywrite (c) Stuart Mitchell 2007
-            """
-            rangeCount = 0
-            objectSense = lp.sense
-            vars=lp.variables()
-            numVars = len(vars)
-            #print "In getCoinMPArrays"
-            #associate each variable with a ordinal
-            v2n=dict(((vars[i],i) for i in range(numVars)))
-            vname2n=dict(((vars[i].name,i) for i in range(numVars)))            
-            n2v=dict((i,vars[i]) for i in range(numVars))
-            #print "After Dictionaries"
-            #objective values
-            NumVarDoubleArray = ctypes.c_double * numVars
-            objectCoeffs=NumVarDoubleArray()
-            print "Get objective Values"
-            for v,val in lp.objective.iteritems():
-                objectCoeffs[v2n[v]]=val
-            #values for variables
-            NumVarStrArray = ctypes.c_char_p * numVars
-            colNames = NumVarStrArray()
-            lowerBounds = NumVarDoubleArray()
-            upperBounds = NumVarDoubleArray()
-            initValues = NumVarDoubleArray()
-            #print "Get Variable information"
-            for v in lp.variables():
-                colNames[v2n[v]] = str(v.name)
-                initValues[v2n[v]] = 0.0
-                if v.lowBound != None:
-                    lowerBounds[v2n[v]] = v.lowBound
-                else:
-                    lowerBounds[v2n[v]] = -coinDblMax
-                if v.upBound != None:
-                    upperBounds[v2n[v]] = v.upBound
-                else:
-                    upperBounds[v2n[v]] = coinDblMax
-            #values for constraints
-            numRows =len(lp.constraints)
-            NumRowDoubleArray = ctypes.c_double * numRows
-            NumRowStrArray = ctypes.c_char_p * numRows
-            NumRowCharArray = ctypes.c_char * numRows
-            coinSense={LpConstraintEQ:"E", LpConstraintLE:"L", LpConstraintGE:"G"}
-            rhsValues = NumRowDoubleArray()
-            rangeValues = NumRowDoubleArray()
-            rowNames = NumRowStrArray()
-            rowType = NumRowCharArray()
-            c2n = {}
-            n2c = {}
-            i = 0
-            print "Get constraint information"
-            for c in lp.constraints:
-                rhsValues[i] = -lp.constraints[c].constant
-                #range values are constraints that are >= a and <=b
-                rangeValues[i] = 0.0
-                rowNames[i] = str(c)
-                rowType[i] = coinSense[lp.constraints[c].sense]
-                c2n[c] = i
-                n2c[i] = c
-                i = i+1
-
-            #return the coefficient matrix as a series of vectors
-            coeffs = lp.coefficients()
-            numels = len(coeffs)  #seems to be all this is good for
-            NumelsDoubleArray = ctypes.c_double * numels
-            NumelsIntArray = ctypes.c_int * (numels)
-            elemBase = NumelsDoubleArray()
-            myelemBase = []
-            indBase = NumelsIntArray()
-            myindBase = []
-            mystartsBase = []
-            mylenBase = []
-            NumVarspIntArray = ctypes.c_int * (numVars + 1)
-            startsBase = NumVarspIntArray()
-            lenBase = NumVarspIntArray()
-            if self.debug: print "get Coefficient information"
-            elements = [[] for i in range(numVars)]
-            for var in coeffs:
-                elements[vname2n[var[0]]].append([var[1],var[2]])
-            if self.debug: print "constructed array"
-                
-            for v in vars:
-                startsBase[v2n[v]] = len (myelemBase)
-                mystartsBase.append(len (myelemBase))
-                myelemBase.extend((var[1] for var in elements[v2n[v]]))
-                myindBase.extend((c2n[var[0]] for var in elements[v2n[v]]))
-                lenBase[v2n[v]] = len(myelemBase) - startsBase[v2n[v]]
-                mylenBase.append(len(myelemBase) - startsBase[v2n[v]])
-            startsBase[numVars] = len(myelemBase)
-            for i in range(numels):
-                elemBase[i]=myelemBase[i]
-                indBase[i]=myindBase[i]
-            #MIP Variables
-            NumVarCharArray = ctypes.c_char * numVars
-            columnType = NumVarCharArray()
-            if lp.isMIP():
-                CoinLpCategories = {LpContinuous: "C",
-                                    LpInteger: "I"}
-                for v in lp.variables():
-                    columnType[v2n[v]] = CoinLpCategories[v.cat]
-            ##print "return all info and solve"
-            return  numVars, numRows, numels, rangeCount, objectSense, objectCoeffs, \
-                rhsValues, rangeValues, rowType, startsBase, lenBase, indBase, \
-                elemBase, lowerBounds, upperBounds, initValues, colNames, \
-                rowNames, columnType, n2v, n2c
-        
+            return lp.status        
     
     COIN = COINMP_DLL
 except (ImportError,OSError):
@@ -1339,7 +1250,7 @@ except (ImportError,OSError):
             return False
         def actualSolve(self, lp):
             """Solve a well formulated lp problem"""
-            raise RuntimeError, "COINMP_DLL: Not Available"
+            raise PulpSolverError, "COINMP_DLL: Not Available"
 
     COIN = COIN_CMD
 
@@ -1384,8 +1295,7 @@ try:
             model = lp.solverModel
             solutionStatus = model.Status
             GRB = gurobipy.GRB
-            gurobiLpStatus = collections.defaultdict(lambda : LpStatusUndefined)
-            gurobiLpStatus.update({GRB.OPTIMAL: LpStatusOptimal,
+            gurobiLpStatus = {GRB.OPTIMAL: LpStatusOptimal,
                                    GRB.INFEASIBLE: LpStatusInfeasible, 
                                    GRB.INF_OR_UNBD: LpStatusInfeasible, 
                                    GRB.UNBOUNDED: LpStatusUnbounded, 
@@ -1395,7 +1305,7 @@ try:
                                    GRB.SOLUTION_LIMIT: LpStatusNotSolved,
                                    GRB.INTERRUPTED: LpStatusNotSolved,
                                    GRB.NUMERIC: LpStatusNotSolved,
-                                   })
+                                   }
             #populate pulp solution values
             for var in lp.variables():
                 try: 
@@ -1421,7 +1331,7 @@ try:
             lp.resolveOK = True
             for var in lp.variables():
                 var.isModified = False
-            lp.status = gurobiLpStatus[solutionStatus]
+            lp.status = gurobiLpStatus.get(solutionStatus, LpStatusUndefined)
             return lp.status
 
         def available(self):
@@ -1446,7 +1356,7 @@ try:
             if lp.sense == LpMaximize:
                 lp.solverModel.setAttr("ModelSense", -1)
             if self.timeLimit:
-                lp.solverModel.setParam("TimeLimit", self.epgap)
+                lp.solverModel.setParam("TimeLimit", self.timeLimit)
             if self.epgap:
                 lp.solverModel.setParam("MIPGap", self.epgap)
             log.debug("add the variables to the problem")
@@ -1477,7 +1387,7 @@ try:
                 elif constraint.sense == LpConstraintEQ:
                     relation = gurobipy.GRB.EQUAL
                 else:
-                    raise RuntimeError, 'Detected an invalid constraint type'
+                    raise PulpSolverError, 'Detected an invalid constraint type'
                 constraint.solverConstraint = lp.solverModel.addConstr(expr, 
                     relation, -constraint.constant, name)
             lp.solverModel.update() 
@@ -1497,7 +1407,31 @@ try:
             solutionStatus = self.findSolutionValues(lp)          
             for var in lp.variables():
                 var.modified = False
+            for constraint in lp.constraints.values():
+                constraint.modified = False
             return solutionStatus
+
+        def actualResolve(self, lp, callback = None):
+            """
+            Solve a well formulated lp problem
+
+            uses the old solver and modifies the rhs of the modified constraints
+            """
+            log.debug("Resolve the Model using gurobi")
+            for constraint in lp.constraints.values():
+                if constraint.modified:
+                    constraint.solverConstraint.setAttr(gurobipy.GRB.Attr.RHS, 
+                                                        -constraint.constant)
+            lp.solverModel.update() 
+            self.callSolver(lp, callback = callback)
+            #get the solution information
+            solutionStatus = self.findSolutionValues(lp)          
+            for var in lp.variables():
+                var.modified = False
+            for constraint in lp.constraints.values():
+                constraint.modified = False
+            return solutionStatus
+
        
 except: #FIXME: Bug because gurobi returns 
         #a gurobi exception on failed imports
@@ -1508,7 +1442,7 @@ except: #FIXME: Bug because gurobi returns
             return False
         def actualSolve(self, lp, callback = None):
             """Solve a well formulated lp problem"""
-            raise RuntimeError, "GUROBI: Not Available"
+            raise PulpSolverError, "GUROBI: Not Available"
 try:
     import ctypes
     def ctypesArrayFill(myList,type = ctypes.c_double):
